#include "config.h"
#include "headers.h"

#define IsInf(x)	((x + x) == (1 + x))
#define IsNaN(x)	((x) != (x))

#define PRINTF (void)printf

#define UNSET	((unsigned int)0xab)

typedef union
{
    double f;
    unsigned char i[8];
} D;

#if defined(HAVE_LONG_DOUBLE)
typedef union
{
    long double f;
    unsigned char i[16];
} E;
#endif

void
show_nanf()
{
    float Inf;
    float zero;

    zero = (float)0.0;
    Inf = zero;
    Inf = (float)1.0/Inf;

    PRINTF("Test of Infinity handling in single-precision IEEE 754 arithmetic:\n");
    PRINTF("\tsizeof(float) = %u\n", (unsigned int)sizeof(float));

    PRINTF("\tNaN in hexadecimal = 0x%08x\n", (int)*((int*)&Inf));

    if (Inf == Inf)	PRINTF("\t(Inf == Inf):\t\tOK\n");
    if (Inf != Inf)	PRINTF("\t(Inf != Inf):\t\tWRONG\n");
    if (Inf <= Inf)	PRINTF("\t(Inf <= Inf):\t\tOK\n");
    if (Inf <  Inf)	PRINTF("\t(Inf <  Inf):\t\tWRONG\n");
    if (Inf >  Inf)	PRINTF("\t(Inf >  Inf):\t\tWRONG\n");
    if (Inf >= Inf)	PRINTF("\t(Inf >= Inf):\t\tOK\n");

    if (Inf == zero)	PRINTF("\t(Inf == 0.0):\t\tWRONG\n");
    if (Inf != zero)	PRINTF("\t(Inf != 0.0):\t\tOK\n");
    if (Inf <= zero)	PRINTF("\t(Inf <= 0.0):\t\tWRONG\n");
    if (Inf <  zero)	PRINTF("\t(Inf <  0.0):\t\tWRONG\n");
    if (Inf >  zero)	PRINTF("\t(Inf >  0.0):\t\tOK\n");
    if (Inf >= zero)	PRINTF("\t(Inf >= 0.0):\t\tOK\n");

    if (zero == Inf)	PRINTF("\t(0.0 == Inf):\t\tWRONG\n");
    if (zero != Inf)	PRINTF("\t(0.0 != Inf):\t\tOK\n");
    if (zero <= Inf)	PRINTF("\t(0.0 <= Inf):\t\tOK\n");
    if (zero <  Inf)	PRINTF("\t(0.0 <  Inf):\t\tOK\n");
    if (zero >  Inf)	PRINTF("\t(0.0 >  Inf):\t\tWRONG\n");
    if (zero >= Inf)	PRINTF("\t(0.0 >= Inf):\t\tWRONG\n");

    if ((-Inf) == (-Inf))	PRINTF("\t(-Inf == -Inf):\t\tOK\n");
    if ((-Inf) != (-Inf))	PRINTF("\t(-Inf != -Inf):\t\tWRONG\n");
    if ((-Inf) <= (-Inf))	PRINTF("\t(-Inf <= -Inf):\t\tOK\n");
    if ((-Inf) <  (-Inf))	PRINTF("\t(-Inf <  -Inf):\t\tWRONG\n");
    if ((-Inf) >  (-Inf))	PRINTF("\t(-Inf >  -Inf):\t\tWRONG\n");
    if ((-Inf) >= (-Inf))	PRINTF("\t(-Inf >= -Inf):\t\tOK\n");

    if ((-Inf) == zero)	PRINTF("\t(-Inf == 0.0):\t\tWRONG\n");
    if ((-Inf) != zero)	PRINTF("\t(-Inf != 0.0):\t\tOK\n");
    if ((-Inf) <= zero)	PRINTF("\t(-Inf <= 0.0):\t\tOK\n");
    if ((-Inf) <  zero)	PRINTF("\t(-Inf <  0.0):\t\tOK\n");
    if ((-Inf) >  zero)	PRINTF("\t(-Inf >  0.0):\t\tWRONG\n");
    if ((-Inf) >= zero)	PRINTF("\t(-Inf >= 0.0):\t\tWRONG\n");

    if (zero == (-Inf))	PRINTF("\t(0.0 == -Inf):\t\tWRONG\n");
    if (zero != (-Inf))	PRINTF("\t(0.0 != -Inf):\t\tOK\n");
    if (zero <= (-Inf))	PRINTF("\t(0.0 <= -Inf):\t\tWRONG\n");
    if (zero <  (-Inf))	PRINTF("\t(0.0 <  -Inf):\t\tWRONG\n");
    if (zero >  (-Inf))	PRINTF("\t(0.0 >  -Inf):\t\tOK\n");
    if (zero >= (-Inf))	PRINTF("\t(0.0 >= -Inf):\t\tOK\n");

    if (zero == ((float)1.0)/Inf)	PRINTF("\t(0.0 == 1/Inf):\t\tOK\n");
    if (zero != ((float)1.0)/Inf)	PRINTF("\t(0.0 != 1/Inf):\t\tWRONG\n");

    if (IsInf(Inf))	PRINTF("\tIsInf(Inf):\t\tOK\n");
    if (!IsInf(Inf))	PRINTF("\t!IsInf(Inf):\t\tWRONG\n");

    if (IsInf(Inf + Inf))	PRINTF("\tIsInf(Inf + Inf):\tOK\n");
    if (!IsInf(Inf + Inf))	PRINTF("\t!IsInf(Inf + Inf):\tWRONG\n");

    if (IsNaN(Inf + Inf))	PRINTF("\tIsNaN(Inf + Inf):\tWRONG\n");
    if (!IsNaN(Inf + Inf))	PRINTF("\t!IsNaN(Inf + Inf):\tOK\n");

    if (IsNaN(Inf - Inf))	PRINTF("\tIsNaN(Inf - Inf):\tOK\n");
    if (!IsNaN(Inf - Inf))	PRINTF("\t!IsNaN(Inf - Inf):\tWRONG\n");

    PRINTF("\n\n");
}


void
show_nan()
{
    double Inf;
    double zero;
    D x;
    int big_endian;
    int k;

    memset(&x.i, (int)UNSET, sizeof(x.i));
    x.f = (double)1.0;
    big_endian = (x.i[0] != 0);

    zero = (double)0.0;
    Inf = zero;
    Inf = (double)1.0/Inf;

    PRINTF("Test of Infinity handling in double-precision IEEE 754 arithmetic:\n");
    PRINTF("\tsizeof(double) = %u\n", (unsigned int)sizeof(double));

    x.f = Inf;
    PRINTF("\tNaN in hexadecimal = 0x");
    if (big_endian)
    {
	for (k = 0; k < sizeof(x.i); ++k)
	{
	    if (x.i[k] == UNSET)
		break;
	    if ((k % 4) == 0)
		PRINTF("_");
	    PRINTF("%02x",(unsigned int)x.i[k]);
	}
	PRINTF("\t[big-endian storage order]");
    }
    else
    {
	int n;

	for (k = sizeof(x.i) - 1; (k > 0) && (x.i[k] == UNSET); --k)
	    /* NO-OP */;

	for (n = 0; k >= 0; --k, ++n)
	{
	    if ((n % 4) == 0)
		PRINTF("_");
	    PRINTF("%02x",(unsigned int)x.i[k]);
	}
	PRINTF("\t[little-endian storage order]");
    }
    PRINTF("\n");

    if (Inf == Inf)	PRINTF("\t(Inf == Inf):\t\tOK\n");
    if (Inf != Inf)	PRINTF("\t(Inf != Inf):\t\tWRONG\n");
    if (Inf <= Inf)	PRINTF("\t(Inf <= Inf):\t\tOK\n");
    if (Inf <  Inf)	PRINTF("\t(Inf <  Inf):\t\tWRONG\n");
    if (Inf >  Inf)	PRINTF("\t(Inf >  Inf):\t\tWRONG\n");
    if (Inf >= Inf)	PRINTF("\t(Inf >= Inf):\t\tOK\n");

    if (Inf == zero)	PRINTF("\t(Inf == 0.0):\t\tWRONG\n");
    if (Inf != zero)	PRINTF("\t(Inf != 0.0):\t\tOK\n");
    if (Inf <= zero)	PRINTF("\t(Inf <= 0.0):\t\tWRONG\n");
    if (Inf <  zero)	PRINTF("\t(Inf <  0.0):\t\tWRONG\n");
    if (Inf >  zero)	PRINTF("\t(Inf >  0.0):\t\tOK\n");
    if (Inf >= zero)	PRINTF("\t(Inf >= 0.0):\t\tOK\n");

    if (zero == Inf)	PRINTF("\t(0.0 == Inf):\t\tWRONG\n");
    if (zero != Inf)	PRINTF("\t(0.0 != Inf):\t\tOK\n");
    if (zero <= Inf)	PRINTF("\t(0.0 <= Inf):\t\tOK\n");
    if (zero <  Inf)	PRINTF("\t(0.0 <  Inf):\t\tOK\n");
    if (zero >  Inf)	PRINTF("\t(0.0 >  Inf):\t\tWRONG\n");
    if (zero >= Inf)	PRINTF("\t(0.0 >= Inf):\t\tWRONG\n");

    if ((-Inf) == (-Inf))	PRINTF("\t(-Inf == -Inf):\t\tOK\n");
    if ((-Inf) != (-Inf))	PRINTF("\t(-Inf != -Inf):\t\tWRONG\n");
    if ((-Inf) <= (-Inf))	PRINTF("\t(-Inf <= -Inf):\t\tOK\n");
    if ((-Inf) <  (-Inf))	PRINTF("\t(-Inf <  -Inf):\t\tWRONG\n");
    if ((-Inf) >  (-Inf))	PRINTF("\t(-Inf >  -Inf):\t\tWRONG\n");
    if ((-Inf) >= (-Inf))	PRINTF("\t(-Inf >= -Inf):\t\tOK\n");

    if ((-Inf) == zero)	PRINTF("\t(-Inf == 0.0):\t\tWRONG\n");
    if ((-Inf) != zero)	PRINTF("\t(-Inf != 0.0):\t\tOK\n");
    if ((-Inf) <= zero)	PRINTF("\t(-Inf <= 0.0):\t\tOK\n");
    if ((-Inf) <  zero)	PRINTF("\t(-Inf <  0.0):\t\tOK\n");
    if ((-Inf) >  zero)	PRINTF("\t(-Inf >  0.0):\t\tWRONG\n");
    if ((-Inf) >= zero)	PRINTF("\t(-Inf >= 0.0):\t\tWRONG\n");

    if (zero == (-Inf))	PRINTF("\t(0.0 == -Inf):\t\tWRONG\n");
    if (zero != (-Inf))	PRINTF("\t(0.0 != -Inf):\t\tOK\n");
    if (zero <= (-Inf))	PRINTF("\t(0.0 <= -Inf):\t\tWRONG\n");
    if (zero <  (-Inf))	PRINTF("\t(0.0 <  -Inf):\t\tWRONG\n");
    if (zero >  (-Inf))	PRINTF("\t(0.0 >  -Inf):\t\tOK\n");
    if (zero >= (-Inf))	PRINTF("\t(0.0 >= -Inf):\t\tOK\n");

    if (zero == ((double)1.0)/Inf)	PRINTF("\t(0.0 == 1/Inf):\t\tOK\n");
    if (zero != ((double)1.0)/Inf)	PRINTF("\t(0.0 != 1/Inf):\t\tWRONG\n");

    if (IsInf(Inf))	PRINTF("\tIsInf(Inf):\t\tOK\n");
    if (!IsInf(Inf))	PRINTF("\t!IsInf(Inf):\t\tWRONG\n");

    if (IsInf(Inf + Inf))	PRINTF("\tIsInf(Inf + Inf):\tOK\n");
    if (!IsInf(Inf + Inf))	PRINTF("\t!IsInf(Inf + Inf):\tWRONG\n");

    if (IsNaN(Inf + Inf))	PRINTF("\tIsNaN(Inf + Inf):\tWRONG\n");
    if (!IsNaN(Inf + Inf))	PRINTF("\t!IsNaN(Inf + Inf):\tOK\n");

    if (IsNaN(Inf - Inf))	PRINTF("\tIsNaN(Inf - Inf):\tOK\n");
    if (!IsNaN(Inf - Inf))	PRINTF("\t!IsNaN(Inf - Inf):\tWRONG\n");

    PRINTF("\n\n");
}


void
show_nanl()
{
#if defined(HAVE_LONG_DOUBLE)
    long double Inf;
    long double zero;
    E x;
    int big_endian;
    int k;

    memset(&x.i, (int)UNSET, sizeof(x.i));
    x.f = (long double)1.0;
    big_endian = (x.i[0] != 0);

    zero = (long double)0.0;
    Inf = zero;
    Inf = (long double)1.0/Inf;

    PRINTF("Test of Infinity handling in extended-precision IEEE 754 arithmetic:\n");
    PRINTF("\tsizeof(long double) = %u\n", (unsigned int)sizeof(long double));

    x.f = Inf;
    PRINTF("\tNaN in hexadecimal = 0x");
    if (big_endian)
    {
	for (k = 0; k < sizeof(x.i); ++k)
	{
	    if (x.i[k] == UNSET)
		break;
	    if ((k % 4) == 0)
		PRINTF("_");
	    PRINTF("%02x",(unsigned int)x.i[k]);
	}
	PRINTF("\t[big-endian storage order]");
    }
    else
    {
	int n;

	for (k = sizeof(x.i) - 1; (k > 0) && (x.i[k] == UNSET); --k)
	    /* NO-OP */;

	for (n = 0; k >= 0; --k, ++n)
	{
	    if ((n % 4) == 0)
		PRINTF("_");
	    PRINTF("%02x",(unsigned int)x.i[k]);
	}
	PRINTF("\t[little-endian storage order]");
    }
    PRINTF("\n");

    if (Inf == Inf)	PRINTF("\t(Inf == Inf):\t\tOK\n");
    if (Inf != Inf)	PRINTF("\t(Inf != Inf):\t\tWRONG\n");
    if (Inf <= Inf)	PRINTF("\t(Inf <= Inf):\t\tOK\n");
    if (Inf <  Inf)	PRINTF("\t(Inf <  Inf):\t\tWRONG\n");
    if (Inf >  Inf)	PRINTF("\t(Inf >  Inf):\t\tWRONG\n");
    if (Inf >= Inf)	PRINTF("\t(Inf >= Inf):\t\tOK\n");

    if (Inf == zero)	PRINTF("\t(Inf == 0.0):\t\tWRONG\n");
    if (Inf != zero)	PRINTF("\t(Inf != 0.0):\t\tOK\n");
    if (Inf <= zero)	PRINTF("\t(Inf <= 0.0):\t\tWRONG\n");
    if (Inf <  zero)	PRINTF("\t(Inf <  0.0):\t\tWRONG\n");
    if (Inf >  zero)	PRINTF("\t(Inf >  0.0):\t\tOK\n");
    if (Inf >= zero)	PRINTF("\t(Inf >= 0.0):\t\tOK\n");

    if (zero == Inf)	PRINTF("\t(0.0 == Inf):\t\tWRONG\n");
    if (zero != Inf)	PRINTF("\t(0.0 != Inf):\t\tOK\n");
    if (zero <= Inf)	PRINTF("\t(0.0 <= Inf):\t\tOK\n");
    if (zero <  Inf)	PRINTF("\t(0.0 <  Inf):\t\tOK\n");
    if (zero >  Inf)	PRINTF("\t(0.0 >  Inf):\t\tWRONG\n");
    if (zero >= Inf)	PRINTF("\t(0.0 >= Inf):\t\tWRONG\n");

    if ((-Inf) == (-Inf))	PRINTF("\t(-Inf == -Inf):\t\tOK\n");
    if ((-Inf) != (-Inf))	PRINTF("\t(-Inf != -Inf):\t\tWRONG\n");
    if ((-Inf) <= (-Inf))	PRINTF("\t(-Inf <= -Inf):\t\tOK\n");
    if ((-Inf) <  (-Inf))	PRINTF("\t(-Inf <  -Inf):\t\tWRONG\n");
    if ((-Inf) >  (-Inf))	PRINTF("\t(-Inf >  -Inf):\t\tWRONG\n");
    if ((-Inf) >= (-Inf))	PRINTF("\t(-Inf >= -Inf):\t\tOK\n");

    if ((-Inf) == zero)	PRINTF("\t(-Inf == 0.0):\t\tWRONG\n");
    if ((-Inf) != zero)	PRINTF("\t(-Inf != 0.0):\t\tOK\n");
    if ((-Inf) <= zero)	PRINTF("\t(-Inf <= 0.0):\t\tOK\n");
    if ((-Inf) <  zero)	PRINTF("\t(-Inf <  0.0):\t\tOK\n");
    if ((-Inf) >  zero)	PRINTF("\t(-Inf >  0.0):\t\tWRONG\n");
    if ((-Inf) >= zero)	PRINTF("\t(-Inf >= 0.0):\t\tWRONG\n");

    if (zero == (-Inf))	PRINTF("\t(0.0 == -Inf):\t\tWRONG\n");
    if (zero != (-Inf))	PRINTF("\t(0.0 != -Inf):\t\tOK\n");
    if (zero <= (-Inf))	PRINTF("\t(0.0 <= -Inf):\t\tWRONG\n");
    if (zero <  (-Inf))	PRINTF("\t(0.0 <  -Inf):\t\tWRONG\n");
    if (zero >  (-Inf))	PRINTF("\t(0.0 >  -Inf):\t\tOK\n");
    if (zero >= (-Inf))	PRINTF("\t(0.0 >= -Inf):\t\tOK\n");

    if (zero == ((long double)1.0)/Inf)	PRINTF("\t(0.0 == 1/Inf):\t\tOK\n");
    if (zero != ((long double)1.0)/Inf)	PRINTF("\t(0.0 != 1/Inf):\t\tWRONG\n");

    if (IsInf(Inf))	PRINTF("\tIsInf(Inf):\t\tOK\n");
    if (!IsInf(Inf))	PRINTF("\t!IsInf(Inf):\t\tWRONG\n");

    if (IsInf(Inf + Inf))	PRINTF("\tIsInf(Inf + Inf):\tOK\n");
    if (!IsInf(Inf + Inf))	PRINTF("\t!IsInf(Inf + Inf):\tWRONG\n");

    if (IsNaN(Inf + Inf))	PRINTF("\tIsNaN(Inf + Inf):\tWRONG\n");
    if (!IsNaN(Inf + Inf))	PRINTF("\t!IsNaN(Inf + Inf):\tOK\n");

    if (IsNaN(Inf - Inf))	PRINTF("\tIsNaN(Inf - Inf):\tOK\n");
    if (!IsNaN(Inf - Inf))	PRINTF("\t!IsNaN(Inf - Inf):\tWRONG\n");

    PRINTF("\n\n");
#endif
}


int
main(void)
{
    show_nanf();
    show_nan();
    show_nanl();

    PRINTF("INF TESTS COMPLETED\n\n");

    return (EXIT_SUCCESS);
}
