### /usr/uumath/src/features/features-1.2.4/make-files.awk, Wed Jul 28 07:32:54 2021
### Edit by Nelson H. F. Beebe <beebe@math.utah.edu>
### ====================================================================
### Create fragments in /tmp/ for configure.ac and miss*.h, using a
### base list of decimal_float function names to generate companion
### lists for other floating-point types.
###
### Usage:
###	awk -f /make-files.awk
###
### [28-Jul-2021]
### ====================================================================

BEGIN {
    initialize() 

    make_miss_h("f")
    make_miss_h("")
    make_miss_h("l")
    make_miss_h("w")
    make_miss_h("q")
    make_miss_h("df")
    make_miss_h("d")
    make_miss_h("dl")
    make_miss_h("d32")
    make_miss_h("d64")
    make_miss_h("d128")

    make_search("f",		"float")
    make_search("",		"double")
    make_search("l",		"long double")
    make_search("w",		"__float80")
    make_search("q",		"__float128")
    make_search("df",		"_Decimal32")
    make_search("d",		"_Decimal64")
    make_search("dl",		"_Decimal128")
    make_search("d32",		"_Decimal32")
    make_search("d64",		"_Decimal64")
    make_search("d128",		"_Decimal128")
    close(SEARCH_FILE)
    print ("See " SEARCH_FILE)

    make_templates("f",		"float")
    make_templates("",		"double")
    make_templates("l",		"long double")
    make_templates("w",		"__float80")
    make_templates("q",		"__float128")
    make_templates("df",	"_Decimal32")
    make_templates("d",		"_Decimal64")
    make_templates("dl",	"_Decimal128")
    make_templates("d32",	"_Decimal32")
    make_templates("d64",	"_Decimal64")
    make_templates("d128",	"_Decimal128")
    close(TEMPLATE_FILE)
    print ("See " TEMPLATE_FILE)

    exit(0)
}

### ====================================================================

function initialize() 
{
    TEMPLATE_FILE = "/tmp/configure.templates"
    system("cat /dev/null > " TEMPLATE_FILE)

    SEARCH_FILE = "/tmp/configure.search"
    system("cat /dev/null > " SEARCH_FILE)

    N = 0
    F[++N] = "acosdegdf"
    F[++N] = "acosdf"
    F[++N] = "acoshdf"
    F[++N] = "asindegdf"
    F[++N] = "asindf"
    F[++N] = "asinhdf"
    F[++N] = "atan2degdf"
    F[++N] = "atan2df"
    F[++N] = "atan2pidf"
    F[++N] = "atandegdf"
    F[++N] = "atandf"
    F[++N] = "atanhdf"
    F[++N] = "cabsdf"
    F[++N] = "cacosdf"
    F[++N] = "cacoshdf"
    F[++N] = "canonicalizedf"
    F[++N] = "cargdf"
    F[++N] = "casindf"
    F[++N] = "casinhdf"
    F[++N] = "catandf"
    F[++N] = "catanhdf"
    F[++N] = "cbrtdf"
    F[++N] = "ccosdf"
    F[++N] = "ceildf"
    F[++N] = "cexpdf"
    F[++N] = "cimagdf"
    F[++N] = "clogdf"
    F[++N] = "compoundndf"
    F[++N] = "conjdf"
    F[++N] = "copysigndf"
    F[++N] = "cosdegdf"
    F[++N] = "cosdf"
    F[++N] = "cospidf"
    F[++N] = "cotandegdf"
    F[++N] = "cpowdf"
    F[++N] = "cprojdf"
    F[++N] = "crealdf"
    F[++N] = "csindf"
    F[++N] = "csinhdf"
    F[++N] = "csoshdf"
    F[++N] = "csqrtdf"
    F[++N] = "ctandf"
    F[++N] = "ctanhdf"
    F[++N] = "erfcdf"
    F[++N] = "erfdf"
    F[++N] = "exp10df"
    F[++N] = "exp10m1df"
    F[++N] = "exp2df"
    F[++N] = "exp2m1df"
    F[++N] = "expdf"
    F[++N] = "expm1df"
    F[++N] = "fabsdf"
    F[++N] = "fdimdf"
    F[++N] = "floordf"
    F[++N] = "fmadf"
    F[++N] = "fmaxdf"
    F[++N] = "fmaxmagdf"
    F[++N] = "fmindf"
    F[++N] = "fminmagdf"
    F[++N] = "fmoddf"
    F[++N] = "frexpdf"
    F[++N] = "fromfpdf"
    F[++N] = "fromfpxdf"
    F[++N] = "gammadf"
    F[++N] = "getpayloaddf"
    F[++N] = "hypotdf"
    F[++N] = "ierfcdf"
    F[++N] = "ierfdf"
    F[++N] = "ilogbdf"
    F[++N] = "j0df"
    F[++N] = "j1df"
    F[++N] = "jndf"
    F[++N] = "ldexpdf"
    F[++N] = "lgammadf"
    F[++N] = "llogbdf"
    F[++N] = "llrintdf"
    F[++N] = "llrounddf"
    F[++N] = "log10df"
    F[++N] = "log10p1df"
    F[++N] = "log1pdf"
    F[++N] = "log2df"
    F[++N] = "log2p1df"
    F[++N] = "logbdf"
    F[++N] = "logdf"
    F[++N] = "logp1df"
    F[++N] = "lrintdf"
    F[++N] = "lrounddf"
    F[++N] = "modfdf"
    F[++N] = "nandf"
    F[++N] = "nearbyintdf"
    F[++N] = "nextafterdf"
    F[++N] = "nextdowndf"
    F[++N] = "nexttowarddf"
    F[++N] = "nextupdf"
    F[++N] = "normalizedf"
    F[++N] = "powdf"
    F[++N] = "powndf"
    F[++N] = "powrdf"
    F[++N] = "psidf"
    F[++N] = "psilndf"
    F[++N] = "qnandf"
    F[++N] = "quantizedf"
    F[++N] = "remainderdf"
    F[++N] = "remquodf"
    F[++N] = "rintdf"
    F[++N] = "rootndf"
    F[++N] = "rounddf"
    F[++N] = "roundevendf"
    F[++N] = "rsqrtdf"
    F[++N] = "samequantumdf"
    F[++N] = "scalbdf"
    F[++N] = "scalblndf"
    F[++N] = "scalbndf"
    F[++N] = "setpayloaddf"
    F[++N] = "setpayloadsigdf"
    F[++N] = "signbitdf"
    F[++N] = "significanddf"
    F[++N] = "sindegdf"
    F[++N] = "sindf"
    F[++N] = "sinpidf"
    F[++N] = "snandf"
    F[++N] = "sqrtdf"
    F[++N] = "strtoddf"
    F[++N] = "tandegdf"
    F[++N] = "tandf"
    F[++N] = "tanpidf"
    F[++N] = "tgammadf"
    F[++N] = "totalorderdf"
    F[++N] = "truncdf"
    F[++N] = "ufromfpdf"
    F[++N] = "ufromfpxdf"
    F[++N] = "y0df"
    F[++N] = "y1df"
    F[++N] = "yndf"
}

function make_miss_h(suffix,    fun, hname, n, outfile)
{
    outfile = ("/tmp/miss_" suffix ".h")
    
    hname = ("HAVE_MISS_" toupper(suffix) "_H")

    print ("/* WARNING: do NOT edit this file: it is auto-generated by make-files.awk */") > outfile
    print ""						>> outfile

    print ("#if !defined(" hname ")")			>> outfile
    print ""						>> outfile
    print ("#define " hname)				>> outfile
    print ""						>> outfile
    print "static const char * miss_" suffix " ="	>> outfile
    print "\t\"\""					>> outfile
    print ""						>> outfile

    for (n = 1; n <= N; ++n)
    {	
	fun = (substr(F[n], 1, length(F[n]) - 2) suffix)
	print ("#if !defined(HAVE_" toupper(fun) ")")	>> outfile
	print ("\t\"" fun " \"")			>> outfile
	print ("#endif")				>> outfile
	print ""					>> outfile
    }

    print ("    ;")					>> outfile
    print ""						>> outfile
    print ("#endif /* !defined(" hname ") */")		>> outfile

    close(outfile)

    print ("See " outfile)
}

function make_search(suffix, type,    outfile)
{
    outfile = SEARCH_FILE

    print ("dnl " type " functions")			>> outfile
    print ""						>> outfile

    for (n = 1; n <= N; ++n)
    {	
	fun = (substr(F[n], 1, length(F[n]) - 2) suffix)
	printf("AC_SEARCH_LIBS(%-23s\t[m],\tAC_DEFINE(%-23s\t[], []))\n", 
	       ("[" fun "],"), 
	       ("[HAVE_" toupper(fun) "],"))		>> outfile
    }

    print ""						>> outfile
}

function make_templates(suffix, type,    outfile)
{
    outfile = TEMPLATE_FILE

    print ("dnl " type " functions")			>> outfile
    print ""						>> outfile

    for (n = 1; n <= N; ++n)
    {	
	fun = (substr(F[n], 1, length(F[n]) - 2) suffix)
	printf("AH_TEMPLATE(HAVE_%-23s\t[Define if %s() is available.])\n", 
	       (toupper(fun) ","), fun)			>> outfile
    }

    print ""						>> outfile
}
